"""Ocean Protocol wrapper around json schema"""

import click
from pathlib import Path
import jsonschema as jsonschema

@click.command()
# @click.option('--source', default="source", help='The schema version filename, must exist in /schemas folder')
# @click.argument('schema', default="metadata_v190118.json")#, help='The schema version filename, must exist in /schemas folder')
@click.argument('schema_file', type=click.Path(exists=True))
# @click.argument('json_file', default="metadata_v190118.json")#, help='The schema version filename, must exist in /schemas folder')
@click.argument('json_file', type=click.Path(exists=True))
# @click.option('--file', default="metadata_v190118.json", help='The schema version filename, must exist in /schemas folder')

def validate(schema_file, json_file):
    """This script validates a json file according a schema file.
    Wraps the jsonschema project, see https://pypi.org/project/jsonschema/.

    Arguments:

        SCHEMA_FILE_NAME: the name of the schema file, found in ./schemas

        JSON_FILE: the relative (to current directory) path of the json file to validate against
    """

    click.echo("schema_file_name: {}".format(schema_file))
    click.echo("json_file {}".format(json_file))
    print(type(schema_file))
    json_file_path = Path.cwd() / json_file
    assert json_file_path.exists(), "Json file path {} does not exist".format(json_file_path)
    schema_file_path = Path.cwd() / schema_file
    assert schema_file_path.exists()

    # Load schema

    # Load file

    validator = Draft4Validator(valid_schema)






if __name__ == "__main__":
    validate()


# %%
from pathlib import Path
import pkg_resources
import logging
import json
import jsonschema as jschema

# %%
# Here is the Schema file
SCHEMA_FILE = Path(pkg_resources.resource_filename('plecos', 'schemas/metadata_190131.json'))
assert SCHEMA_FILE.exists(), "Can't find schema file {}".format(SCHEMA_FILE)


# %%
def load_json(json_file_path):
    assert Path(json_file_path).exists(), "File path {} does not exist".format(json_file_path)
    with open(json_file_path) as fp:
       json_dict  = json.load(fp)
    return json_dict

def validate_against(this_json_file, schema_file):
    """
    """
    logging.info("Schema: {}".format(schema_file))
    this_json_schema_dict = load_json(schema_file)
    logging.info("Json to validate: {}".format(this_json_file))
    this_json_dict = load_json(this_json_file)
    # assert Path(schema_file).exists(), "Schema file path {} does not exist".format(schema_file)
    # with open(schema_file) as f_schema:
    #     this_json_schema_dict = json.load(f_schema)


    # assert Path(this_json_file).exists(), "Json file path {} does not exist".format(this_json_file)
    # with open(this_json_file) as f_json:
    #     this_json = json.load(f_json)

    validator = jschema.validators.Draft7Validator(this_json_schema_dict)
    return validator.validate(this_json_dict)

    # logging.info("Schema: {}".format(schema_file))
    # logging.info("Json to validate: {}".format(json_file))
    # print(type(schema_file))
    # json_file_path = Path.cwd() / json_file
    # assert json_file_path.exists(), "Json file path {} does not exist".format(json_file_path)
    # schema_file_path = Path.cwd() / schema_file
    # assert schema_file_path.exists()

def is_valid(json_file_abs_path, schema_file=SCHEMA_FILE):
    logging.info("Schema: {}".format(schema_file))
    this_json_schema_dict = load_json(schema_file)
    logging.info("Json to validate: {}".format(json_file_abs_path))
    this_json_dict = load_json(json_file_abs_path)

    validator = jschema.validators.Draft7Validator(this_json_schema_dict)
    # validator = jschema.validators.Draft7Validator(this_json_schema_dict)
    return validator.is_valid(this_json_dict)


def validate(json_file_abs_path, schema_file=SCHEMA_FILE):
    """ Wrapper around validate_against

    TODO: This function should handle default schemas for DDO and MetaData

    :param json_file_abs_path:
    :param schema_file:
    :return:
    """
    return validate_against(json_file_abs_path, schema_file)

def list_errors(json_file_abs_path, schema_file=SCHEMA_FILE):
    """ Iterate over the validation errors, print to log.warn

    :param json_file_abs_path:
    :param schema_file:
    :return:
    """
    logging.info("Schema: {}".format(schema_file))
    this_json_schema_dict = load_json(schema_file)
    logging.info("Json to validate: {}".format(json_file_abs_path))
    this_json_dict = load_json(json_file_abs_path)

    validator = jschema.Draft4Validator(this_json_schema_dict)
    logging.info("Instantiated validator {}".format(validator))

    errors = sorted(validator.iter_errors(this_json_dict), key=lambda e: e.path)
    for i,err in enumerate(errors):
        stack_path = list(err.relative_path)
        stack_path = [str(p) for p in stack_path]
        logging.warning("Error {} at {}".format(i,"/".join(stack_path)))
        logging.warning("\t" + err.message)




raise

# from json_versions import json1, json2, json4, meta_data_links
from schema_definitions import valid_schema
import requests

from jsonschema import validate, ErrorTree, exceptions
from jsonschema.validators import Draft4Validator


# %%
# Define validator and use pre-defined schema
validator = Draft4Validator(valid_schema)


# %%
# parse urls in  and return list which contains metadata as json
def get_json_from_links(meta_list):
    list_meta_2 = []
    for i in meta_list:
        i = requests.get(i)

        if i.status_code != 200:
            continue
        elif i.status_code == 200:
            list_meta_2.append(i.json())

    print(list_meta_2)


# get_json_from_links(meta_data_links)


# %%
# Return errors in a list
def validate_single_json(single_json):
    error_list = []
    error_path_list = []

    if validator.is_valid(single_json):
        print("is valid")

    else:
        errors = sorted(validator.iter_errors(single_json), key=lambda e: e.path)
        for error in errors:

            error = error.message
            error_list.append(error)

        print(error_list)
        # print(error_path_list)

        # this returns the error paths in a list
        for error_path in errors:
            error_path = error_path.path

            error_path_list.append(error_path)

        print(error_path_list)


# validate_single_json(json2)


# %%
# validate multiple assets within a list
def validate_multiple_json(link_list):
    list_meta_2 = []
    for i in link_list:
        i = requests.get(i)

        if i.status_code != 200:
            continue
        elif i.status_code == 200:
            list_meta_2.append(i.json())

    for x in list_meta_2:

        if validator.is_valid(x):
            print(" is valid \n ")

        else:
            error_list = []
            error_path_list = []
            errors = sorted(validator.iter_errors(x), key=lambda e: e.path)

            # this returns all validation errors in a list
            for error in errors:
                error = error.message

                error_list.append(error)

            print(error_list)

            # this returns the error paths in a list
            for error_path in errors:

                error_path = error_path.path

                error_path_list.append(error_path)

            print("{} \n".format(error_path_list))

            # TODO: if valid return true


# validate_multiple_json(meta_data_links)



